#include "papu_internals.hh"
#include "papu_squareunit.hh"

void squaregenunit::write(fBYTE ind, fBYTE value)
{
	#if PAPU_SQUARE_DEBUG
    fprintf(stderr, "square.write(%u, 0x%02X) - ", ind, value);
    #endif
    waveunit::write(ind, value);
    decayunit::write(ind, value);
    lengthunit::write(ind, value);
    
    if(ind == 1)
    {
        sweepdata = value;
        sweepspeed = ((value >> 4) & 7) + 1;
        sweepshift = value & 7;
        sweepenabled = (value & 0x87) > 0x80;
        #if PAPU_SQUARE_DEBUG
        fprintf(stderr,
          "{sweepshift %d %s, rate %d, %sabled}",
          sweepshift, (value&8)?"down":"up", (value>>4)&7, (value&128)?"en":"dis");
        #endif
    }
    else if(ind == 0)
    {
        static const BYTE dutyvalues[4] = {2,4,8,12};
    	#if PAPU_SQUARE_DEBUG
        fprintf(stderr, "{duty=%d)\n", value >> 6);
        #endif
        dutytype = dutyvalues[value >> 6];
    }
    jammed = getwavelen() < 8;
    #if PAPU_SQUARE_DEBUG
    if(jammed)
    	fprintf(stderr, " -> jammed\n");
    else if(silent())
    	fprintf(stderr, " -> silent\n");
    else
    	fprintf(stderr, " -> ok\n");
	#endif
}

BEGIN_HZTICKER(120, squaregenunit)
    if(!sweeppos)
    {
        // Update sweep speed counter
        sweeppos = sweepspeed;
        if(sweepenabled /* && !silent() */) // May not update if silent!
        {
            unsigned len = getwavelen();
            if(sweepdata & 15)
                len -= (len >> sweepshift) + decr;
            else
                len += (len >> sweepshift);
            if(setwavelen(len) || len < 8)
                jammed = true;
        }
    }
    else
        --sweeppos;
END_HZTICKER
